==> wrogue-0.7.4.tar.bz2: wrogue-0.7.4/src/career.c <==
/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Career
 * Description: -
 */

#define Uses_Ui
#define Uses_Util
#define Uses_DataFile
#define Uses_ProgramManager
#define Uses_Object
#define Uses_Stats
#define Uses_Perks
#define Uses_Faction
#define Uses_Inventory
#include "mheader.h"

#include "career.h"


#define N_CAREER_TYPES			2

#define CAREER_BOX_BUFFER_SIZE		50



#if defined(DEBUG)

static void		career_validate(const CAREER *);

#endif




/*
 * career type names
 */
static const char *	CareerTypeName[N_CAREER_TYPES] = {
	
	"Basic",
	"Advanced"
	
};



/*
 * the career box
 */
static CAREER  *	CareerBox = NULL;
static N_CAREERS	CareerBoxCurrentSize = 0;
static N_CAREERS	CareerBoxMaxSize = 0;




/*
 * career template (default values)
 */
static const CAREER CareerTemplate = {

	/* NAME */ "", /* TYPE */ CAREER_TYPE_BASIC, /* COLOR */ C_DEFAULT,
	/* MAX STAT ADVANCE */ {0, 0, 0, 0, 0, 0, 0, 0, 0},
	/* PERKS */ {false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false},
	/* TRAPPINGS */ NULL, /* EXITS */ {"", "", "", "", ""}

};



/*
 * career module clean up
 */
void career_clean_up(void)
{
	CAREER_COUNTER i;

	if (CareerBox == NULL) {

		return;
	}

	for (i = 0; i < CareerBoxCurrentSize; i++) {

		list_destroy_with(CareerBox[i].trappings,
			object_destroy
		);
	}

	free(CareerBox);
}



/*
 * adds a career to the career box
 */
CAREER * career_box_new_career(void)
{
	CAREER *career;

	++CareerBoxCurrentSize;

	if (CareerBoxCurrentSize > CareerBoxMaxSize) {
		CareerBoxMaxSize += CAREER_BOX_BUFFER_SIZE;

		CareerBox = checked_realloc(CareerBox,
			CareerBoxMaxSize *
			sizeof *CareerBox
		);
	}

	career = &CareerBox[CareerBoxCurrentSize - 1];

	*career = CareerTemplate;

	career->trappings = list_new();

	return career;
}



/*
 * optimizes the career box
 */
void career_box_optimize(void)
{

	if (CareerBoxCurrentSize < CareerBoxMaxSize) {
		CareerBoxMaxSize = CareerBoxCurrentSize;

		CareerBox = checked_realloc(CareerBox,
			CareerBoxMaxSize *
			sizeof *CareerBox
		);
	}
}



/*
 * sets a character's career
 */
void career_set(CHARACTER *character, CAREER_INDEX career_index)
{
	const CAREER *career;
	
	career = get_career_pointer(career_index);
	
	character->career = career_index;
	
	character->color = career->color;
}



/*
 * returns a career pointer
 */
CAREER * get_career_pointer(CAREER_INDEX career_index)
{

	if (career_index == CAREER_NIL) {

		return NULL;
	}

	return &CareerBox[career_index];
}



/*
 * returns the number of available careers
 */
N_CAREERS get_n_careers(void)
{

	return CareerBoxCurrentSize;
}



/*
 * checks whether a character has completed his current career or not
 */
bool career_completed(const CHARACTER *character)
{
	STAT_COUNTER i;
	CAREER *career;

	career = get_career_pointer(character->career);

	for (i = 0; i < MAX_STATS; i++) {

		if (character->stat[i].advance <
			career->max_stat_advance[i]) {

			return false;
		}
	}

	return true;
}



/*
 * the "Career" screen
 */
void career_screen(const CAREER *career)
{

	command_bar_set(1, CM_EXIT);

	render_career_screen(career);

	update_screen();

	command_bar_get_command();
}



/*
 * returns the description of a career
 */
char * career_description(char *description, const CAREER *career)
{

	return data_file_career_description(description, career);
}



/*
 * hands out the trappings of a career to the passed character
 */
void career_give_trappings(CHARACTER *character)
{
	const CAREER *career;
	OBJECT *object;
	LIST_NODE *node;

	career = get_career_pointer(character->career);

	for (node = career->trappings->head;
		node != NULL;
		node = node->next) {

		object = object_clone((OBJECT *)node->data);

		inventory_add(character, object);
	}
}



/*
 * name -> career index
 */
CAREER_INDEX name_to_career_index(const char *name)
{
	CAREER_COUNTER i;

	for (i = 0; i < CareerBoxCurrentSize; i++) {

		if (strings_equal(name, CareerBox[i].name)) {

			return i;
		}
	}

	die("*** CORE ERROR *** invalid career (%s)", name);

	return 0;
}



/*
 * returns the name of a career type
 */
const char * career_type_name(CAREER_TYPE career_type)
{

	return CareerTypeName[career_type];
}



/*
 * name -> career type
 */
CAREER_TYPE name_to_career_type(const char *name)
{
	CAREER_TYPE_COUNTER i;

	for (i = 0; i < N_CAREER_TYPES; i++) {

		if (strings_equal(name, CareerTypeName[i])) {

			return i;
		}
	}

	die("*** CORE ERROR *** invalid career type (%s)", name);

	return 0;
}


#if defined(DEBUG)

/*
 * validates the content of the career box
 */
void career_box_validate(void)
{
	CAREER_COUNTER i;

	for (i = 0; i < CareerBoxCurrentSize; i++) {
		const CAREER *career;
		
		career = &CareerBox[i];
		
		career_validate(career);
	}
}

#endif


#if defined(DEBUG)

/*
 * validates a career
 */
static void career_validate(const CAREER *career)
{
	CAREER_EXIT_COUNTER i;
	
	for (i = 0; i < MAX_CAREER_EXITS; i++) {
		
		if (is_empty_string(career->exit[i])) {
			
			break;
		}
		
		name_to_career_index(career->exit[i]);
	}
}

#endif

==> wrogue-0.7.5.tar.bz2: wrogue-0.7.5/src/career.c <==
/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Career
 * Description: -
 */

#define Uses_Ui
#define Uses_Util
#define Uses_DataFile
#define Uses_ProgramManager
#define Uses_Object
#define Uses_Stats
#define Uses_Perks
#define Uses_Faction
#define Uses_Inventory
#include "mheader.h"

#include "career.h"


#define N_CAREER_TYPES			2

#define CAREER_BOX_BUFFER_SIZE		32



#if defined(DEBUG)

static void		career_validate(const CAREER *);

#endif

static void *		career_new(void);
static void		career_free(void *);





/*
 * career type names
 */
static const char *	CareerTypeName[N_CAREER_TYPES] = {
	
	"Basic",
	"Advanced"
	
};



/*
 * the career box
 */
static BOX  *		CareerBox = NULL;




/*
 * career template (default values)
 */
static const CAREER CareerTemplate = {

	/* NAME */ "", /* TYPE */ CAREER_TYPE_BASIC, /* COLOR */ C_DEFAULT,
	/* MAX STAT ADVANCE */ {0, 0, 0, 0, 0, 0, 0, 0, 0},
	/* PERKS */ {false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false,
	false,false,false,false,false,false,false,false,false,false,
	false,false,false,false,false,false,false,false},
	/* TRAPPINGS */ NULL, /* EXITS */ {"", "", "", "", ""}

};



/*
 * Career module init
 */
void career_init(void)
{

	CareerBox = box_new(career_new, career_free, 
		CAREER_BOX_BUFFER_SIZE
	);
}



/*
 * Career module clean up
 */
void career_clean_up(void)
{

	if (CareerBox != NULL) {

		box_free(CareerBox);

		return;
	}
}



/*
 * adds a career to the career box
 */
CAREER * career_box_new_career(void)
{

	return box_new_item(CareerBox);
}



/*
 * optimizes the career box
 */
void career_box_optimize(void)
{

	box_optimize(CareerBox);
}



/*
 * sets a character's career
 */
void career_set(CHARACTER *character, CAREER_INDEX career_index)
{
	const CAREER *career;
	
	career = get_career_pointer(career_index);
	
	character->career = career_index;
	
	character->color = career->color;
}



/*
 * returns a career pointer
 */
CAREER * get_career_pointer(CAREER_INDEX career_index)
{

	if (career_index == CAREER_NIL) {

		return NULL;
	}

	return CareerBox->item[career_index];
}



/*
 * returns the number of available careers
 */
N_CAREERS get_n_careers(void)
{

	return CareerBox->current_size;
}



/*
 * checks whether a character has completed his current career or not
 */
bool career_completed(const CHARACTER *character)
{
	STAT i;
	CAREER *career;

	career = get_career_pointer(character->career);

	for (i = 0; i < MAX_STATS; i++) {

		if (character->stat[i].advance <
			career->max_stat_advance[i]) {

			return false;
		}
	}

	return true;
}



/*
 * the "Career" screen
 */
void career_screen(const CAREER *career)
{

	command_bar_set(1, CM_EXIT);

	render_career_screen(career);

	update_screen();

	command_bar_get_command();
}



/*
 * returns the description of a career
 */
char * career_description(char *description, const CAREER *career)
{

	return data_file_career_description(description, career);
}



/*
 * hands out the trappings of a career to the passed character
 */
void career_give_trappings(CHARACTER *character)
{
	const CAREER *career;
	OBJECT *object;
	LIST_NODE *node;

	career = get_career_pointer(character->career);

	for (node = career->trappings->head;
		node != NULL;
		node = node->next) {

		object = object_clone((OBJECT *)node->data);

		inventory_add(character, object);
	}
}



/*
 * name -> career index
 */
CAREER_INDEX name_to_career_index(const char *name)
{
	CAREER_INDEX i;

	for (i = 0; i < CareerBox->current_size; i++) {
		const CAREER *career;
		
		career = CareerBox->item[i];

		if (strings_equal(name, career->name)) {

			return i;
		}
	}

	die("*** CORE ERROR *** invalid career (%s)", name);

	return 0;
}



/*
 * returns the name of a career type
 */
const char * career_type_name(CAREER_TYPE career_type)
{

	return CareerTypeName[career_type];
}



/*
 * name -> career type
 */
CAREER_TYPE name_to_career_type(const char *name)
{
	CAREER_TYPE i;

	for (i = 0; i < N_CAREER_TYPES; i++) {

		if (strings_equal(name, CareerTypeName[i])) {

			return i;
		}
	}

	die("*** CORE ERROR *** invalid career type (%s)", name);

	return 0;
}


#if defined(DEBUG)

/*
 * validates the content of the career box
 */
void career_box_validate(void)
{
	CAREER_INDEX i;

	for (i = 0; i < CareerBox->current_size; i++) {
		const CAREER *career;
		
		career = CareerBox->item[i];
		
		career_validate(career);
	}
}

#endif


#if defined(DEBUG)

/*
 * validates a career
 */
static void career_validate(const CAREER *career)
{
	CAREER_EXIT_INDEX i;
	
	for (i = 0; i < MAX_CAREER_EXITS; i++) {
		
		if (is_empty_string(career->exit[i])) {
			
			break;
		}
		
		name_to_career_index(career->exit[i]);
	}
}

#endif



/*
 * allocs a career
 */
static void * career_new(void)
{
	CAREER *career;

	career = checked_malloc(sizeof *career);

	*career = CareerTemplate;

	career->trappings = list_new();

	return career;
}



/*
 * frees a career
 */
static void career_free(void *data)
{
	CAREER *career;
	
	career = (CAREER *)data;

	list_destroy_with(career->trappings,
		object_destroy
	);

	free(career);
}
